<?php
/**
* Price increment/reduction by groups, categories and more
*
* NOTICE OF LICENSE
*
* This product is licensed for one customer to use on one installation (test stores and multishop included).
* Site developer has the right to modify this module to suit their needs, but can not redistribute the module in
* whole or in part. Any other use of this module constitues a violation of the user agreement.
*
* DISCLAIMER
*
* NO WARRANTIES OF DATA SAFETY OR MODULE SECURITY
* ARE EXPRESSED OR IMPLIED. USE THIS MODULE IN ACCORDANCE
* WITH YOUR MERCHANT AGREEMENT, KNOWING THAT VIOLATIONS OF
* PCI COMPLIANCY OR A DATA BREACH CAN COST THOUSANDS OF DOLLARS
* IN FINES AND DAMAGE A STORES REPUTATION. USE AT YOUR OWN RISK.
*
*  @author    idnovate.com <info@idnovate.com>
*  @copyright 2016 idnovate.com
*  @license   See above
*/

class GroupincConfiguration extends ObjectModel
{
    public $id_groupinc_configuration;
    public $name;
    public $type;
    public $mode;
    public $price_calculation;
    public $price_application;
    public $fix;
    public $percentage;
    public $min_result_price;
    public $max_result_price;
    public $threshold_min_price;
    public $threshold_max_price;
    public $threshold_price;
    public $skip_discounts = false;
    public $override_discounts = false;
    public $groups;
    public $customers;
    public $countries;
    public $zones;
    public $categories;
    public $products;
    public $manufacturers;
    public $suppliers;
    public $currencies;
    public $languages;
    public $active = true;
    public $show_as_discount = false;
    public $backoffice = false;
    public $priority;
    public $first_condition = false;
    public $id_shop;
    public $date_add;
    public $date_upd;
    public $date_from;
    public $date_to;
    public $features;
    public $attributes;
    public $product_qty;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'groupinc_configuration',
        'primary' => 'id_groupinc_configuration',
        'multilang' => false,
        'fields' => array(
            'name' =>                   array('type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'size' => 100),
            'type' =>                   array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'mode' =>                   array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'price_calculation' =>      array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'price_application' =>      array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'fix' =>                    array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'percentage' =>             array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'min_result_price' =>       array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'max_result_price' =>       array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'threshold_min_price' =>    array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'threshold_max_price' =>    array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'threshold_price' =>        array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'product_qty' =>            array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'skip_discounts' =>         array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'override_discounts' =>     array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'groups' =>                 array('type' => self::TYPE_STRING),
            'countries' =>              array('type' => self::TYPE_STRING),
            'products' =>               array('type' => self::TYPE_STRING),
            'customers' =>              array('type' => self::TYPE_STRING),
            'zones' =>                  array('type' => self::TYPE_STRING),
            'categories' =>             array('type' => self::TYPE_STRING),
            'manufacturers' =>          array('type' => self::TYPE_STRING),
            'currencies' =>             array('type' => self::TYPE_STRING),
            'languages' =>              array('type' => self::TYPE_STRING),
            'suppliers' =>              array('type' => self::TYPE_STRING),
            'features' =>               array('type' => self::TYPE_STRING),
            'attributes' =>             array('type' => self::TYPE_STRING),
            'active' =>                 array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'show_as_discount' =>       array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'backoffice' =>             array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'first_condition' =>        array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'copy_post' => false),
            'priority' =>               array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'copy_post' => false),
            'id_shop' =>                array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'copy_post' => false),
            'date_add' =>               array('type' => self::TYPE_DATE, 'validate' => 'isDate', 'copy_post' => false),
            'date_upd' =>               array('type' => self::TYPE_DATE, 'validate' => 'isDate', 'copy_post' => false),
            'date_from' =>               array('type' => self::TYPE_DATE, 'copy_post' => false),
            'date_to' =>               array('type' => self::TYPE_DATE, 'copy_post' => false),

        ),
    );

    public function __construct($id = null, $id_lang = null)
    {
        parent::__construct($id, $id_lang);
    }

    public function add($autodate = true, $null_values = true)
    {
        $this->id_shop = ($this->id_shop) ? $this->id_shop : Context::getContext()->shop->id;
        $success = parent::add($autodate, $null_values);
        return $success;
    }

    public function toggleStatus()
    {
        parent::toggleStatus();
        return Db::getInstance()->execute('
        UPDATE `'._DB_PREFIX_.bqSQL($this->def['table']).'`
        SET `date_upd` = NOW()
        WHERE `'.bqSQL($this->def['primary']).'` = '.(int)$this->id);
    }

    public function delete()
    {
        if (parent::delete()) {
            return $this->deleteImage();
        }
    }

    public static function getAdminGIconfigurations($id_shop, $id_product)
    {
        $today = date("Y-m-d H:i:s");

        $query = '
                SELECT gi.* FROM `'._DB_PREFIX_.'groupinc_configuration` gi WHERE gi.`id_shop` = '
                .(int)$id_shop.' AND gi.`active` = 1 AND gi.`backoffice` = 1
                AND (date_from <= "'.$today. '" OR date_from = "0000-00-00 00:00:00")
                AND (date_to >= "'.$today.'" OR date_to = "0000-00-00 00:00:00")
                ORDER BY gi.`priority`,gi.`id_groupinc_configuration`';

        $categories = Product::getProductCategories($id_product);
        $product = new Product($id_product);
        $id_manufacturer = $product->id_manufacturer;
        $id_supplier = $product->id_supplier;

        $giconfigurations = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
        if ($giconfigurations === false) {
            return false;
        }

        $array_configurations_result = array();
        foreach ($giconfigurations as $conf) {
            /* retrocompatibility with old rules which have the value 'all' in the database */
            if ($conf['products'] == 'all') {
                $conf['products'] = '';
            }
            if ($conf['categories'] == 'all') {
                $conf['categories'] = '';
            }
            if ($conf['manufacturers'] == 'all') {
                $conf['manufacturers'] = '';
            }
            if ($conf['suppliers'] == 'all') {
                $conf['suppliers'] = '';
            }

            if ($conf['products'] == '' && $conf['categories'] == '' && $conf['manufacturers'] == '' && $conf['suppliers'] == '') {
                $array_configurations_result[] = $conf;
                if ($conf['first_condition']) {
                    break;
                } else {
                    continue;
                }
            }
            $filter_categories = true;
            $filter_products = true;

            if (@unserialize($conf['categories']) !== false) {
                $categories_array = unserialize($conf['categories']);
            } else {
                $categories_array = explode(';', $conf['categories']);
            }

            if ($conf['categories'] !== '' && $conf['products'] == '') {
                foreach ($categories as $category) {
                    if (in_array($category, $categories_array)) {
                        $filter_categories = true;
                        $filter_products = true;
                        break;
                    } else {
                        $filter_categories = false;
                    }
                }
                if (!$filter_categories) {
                    $filter_products = false;
                }
            } else if ($conf['categories'] == '' && $conf['products'] !== '') {
                $products_array = explode(';', $conf['products']);
                if (!in_array($id_product, $products_array)) {
                    $filter_products = false;
                    $filter_categories = true;
                }
            } else if ($conf['categories'] !== '' && $conf['products'] !== '') {
                foreach ($categories as $category) {
                    if (!in_array($category, $categories_array)) {
                        $filter_categories = false;
                    } else {
                        $filter_categories = true;
                        break;
                    }
                }
                if (!$filter_categories) {
                    $products_array = explode(';', $conf['products']);
                    if (!in_array($id_product, $products_array)) {
                        $filter_products = false;
                    } else {
                        $filter_products = true;
                    }
                } else {
                    $products_array = explode(';', $conf['products']);
                    if (!in_array($id_product, $products_array)) {
                        $filter_products = false;
                    }
                }
            }

            $filter_manufacturers = true;
            if ($conf['manufacturers'] !== '') {
                $manufacturers_array = explode(';', $conf['manufacturers']);
                if (!in_array($id_manufacturer, $manufacturers_array)) {
                    $filter_manufacturers = false;
                }
            }
            $filter_suppliers = true;
            if ($conf['suppliers'] !== '') {
                $suppliers_array = explode(';', $conf['suppliers']);
                if (!in_array($id_supplier, $suppliers_array)) {
                    $filter_suppliers = false;
                }
            }

            if ($filter_categories && $filter_products && $filter_manufacturers && $filter_suppliers) {
                $array_configurations_result[] = $conf;
                if ($conf['first_condition']) {
                    break;
                } else {
                    continue;
                }
            }
        }
        /*var_dump($filter_groups);
        var_dump($filter_customers);
        var_dump($filter_countries);
        var_dump($filter_zones);
        var_dump($filter_categories);
        var_dump($filter_products);
        var_dump($filter_manufacturers);
        var_dump($filter_suppliers);*/
        //d($array_configurations_result);

        if (count($array_configurations_result) > 0) {
            return $array_configurations_result;
        } else {
            return false;
        }
    }

    public static function getGIConfigurations($id_shop, $id_product, $id_customer, $id_country, $id_state, $id_currency, $id_lang, $taxes, $discounts = false, $id_product_attribute = 0)
    {
        $query = '';
        $today = date("Y-m-d H:i:s");

        if (!$discounts) {
            if ($taxes) {
                $applications_taxes = array(2,3);
            } else {
                $applications_taxes = array(0,1);
            }
            $query = '
                    SELECT gi.* FROM `'._DB_PREFIX_.'groupinc_configuration` gi WHERE gi.`id_shop` = '.(int)$id_shop.'
                    AND gi.`active` = 1 AND gi.`price_application` in ('.implode(",", $applications_taxes).') AND gi.`show_as_discount` <> 1';
        } else {
            $query = '
                    SELECT gi.* FROM `'._DB_PREFIX_.'groupinc_configuration` gi WHERE gi.`id_shop` = '.(int)$id_shop.'
                    AND gi.`active` = 1 AND gi.`mode` = 1 AND gi.`show_as_discount` = 1 ';
        }
        $datefilters = ' AND (date_from <= "'.$today. '" OR date_from = "0000-00-00 00:00:00") AND (date_to >= "'.$today.'" OR date_to = "0000-00-00 00:00:00")';

        $query = $query.$datefilters;

        $orderby = ' ORDER BY gi.`priority`, gi.`id_groupinc_configuration` ASC';

        $query = $query.$orderby;

        $giconfigurations = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

        if ($giconfigurations === false) {
            return false;
        }

        $array_configurations_result = array();
        $customer = new Customer($id_customer);
        $customer_groups = $customer->getGroupsStatic($customer->id);
        $categories = Product::getProductCategories($id_product);
        $country = new Country($id_country);

        $product = new Product($id_product);
        $product_attribute_combinations = $product->getAttributeCombinationsById($id_product_attribute, $id_lang);
        $id_manufacturer = $product->id_manufacturer;
        $id_supplier = $product->id_supplier;
        $zone = 0;

        if ($id_state > 0) {
            $zone = State::getIdZone($id_state);
        } else if ($id_country != null and $id_country > 0) {
            $zone = $country->getIdZone($id_country);
        }

        $cart = Context::getContext()->cart;
        $products = array();
        if (!empty($cart) && $cart->id) {
            $products = GroupincConfiguration::getProductsFromIdCart($cart->id);
        }

        $product_count_flag = false;

        foreach ($giconfigurations as $conf) {
            if ($conf['product_qty'] > 0) {
                if (!empty($products)) {
                    foreach ($products as $product) {
                        if ($product['id_product'] == $id_product) {
                            if ((int)$product['quantity'] >= (int)$conf['product_qty']) {
                                $product_count_flag = true;
                                break;
                            }
                        }
                    }
                }
            } else {
                $product_count_flag = true;
            }

            if (!$product_count_flag) {
                continue;
            }

            /* retrocompatibility with old rules which have the value 'all' in the database */
            if ($conf['currencies'] == 'all') {
                $conf['currencies'] = '';
            }
            if ($conf['languages'] == 'all') {
                $conf['languages'] = '';
            }
            if ($conf['groups'] == 'all') {
                $conf['groups'] = '';
            }
            if ($conf['products'] == 'all') {
                $conf['products'] = '';
            }
            if ($conf['customers'] == 'all') {
                $conf['customers'] = '';
            }
            if ($conf['countries'] == 'all') {
                $conf['countries'] = '';
            }
            if ($conf['zones'] == 'all') {
                $conf['zones'] = '';
            }
            if ($conf['categories'] == 'all') {
                $conf['categories'] = '';
            }
            if ($conf['manufacturers'] == 'all') {
                $conf['manufacturers'] = '';
            }
            if ($conf['suppliers'] == 'all') {
                $conf['suppliers'] = '';
            }
            if ($conf['features'] == 'all' || empty($conf['features'])) {
                $conf['features'] = '';
            }
            if ($conf['attributes'] == 'all' || empty($conf['attributes'])) {
                $conf['attributes'] = '';
            }


            if ($product_count_flag && $conf['attributes'] == '' && $conf['features'] == '' && $conf['currencies'] == '' && $conf['languages'] == '' && $conf['groups'] == '' && $conf['products'] == '' && $conf['customers'] == '' && $conf['countries'] == '' && $conf['zones'] == '' && $conf['categories'] == '' && $conf['manufacturers'] == '' && $conf['suppliers'] == '') {
                $array_configurations_result[] = $conf;
                if ($conf['first_condition']) {
                    break;
                } else {
                    continue;
                }
            }

            $filter_features = false;
            $array_features_selected = Tools::jsonDecode($conf['features'], true);
            $product_features = Product::getFeaturesStatic((int)$id_product);

            $flag_features = 0;
            if (!empty($array_features_selected)) {
                foreach ($product_features as $pf) {
                    if (isset($array_features_selected[$pf['id_feature']])) {
                        $array_f = explode(";", $array_features_selected[$pf['id_feature']]);
                        if (in_array($pf['id_feature_value'], $array_f)) {
                            $flag_features++;
                            continue;
                        }
                    }
                }
            } else {
                $filter_features = true;
            }

            if ($flag_features == count($product_features)) {
                $filter_features = true;
            }

            $filter_attributes = false;
            $array_attributes_selected = Tools::jsonDecode($conf['attributes'], true);

            if (!empty($array_attributes_selected)) {
                foreach ($product_attribute_combinations as $key => $prod_attr_comb) {
                    if (isset($array_attributes_selected[$prod_attr_comb['id_attribute_group']])) {
                        $array_a = explode(";", $array_attributes_selected[$prod_attr_comb['id_attribute_group']]);
                        if (in_array($prod_attr_comb['id_attribute'], $array_a)) {
                            $filter_attributes = true;
                        } else {
                            $filter_attributes = false;
                            break;
                        }
                    }
                }
            } else {
                $filter_attributes = true;
            }

            $filter_currencies = true;
            if ($conf['currencies'] !== '') {
                $currencies_array = explode(';', $conf['currencies']);
                if (!in_array($id_currency, $currencies_array)) {
                    $filter_currencies = false;
                }
            }
            $filter_languages = true;
            if ($conf['languages'] !== '') {
                $languages_array = explode(';', $conf['languages']);
                if (!in_array($id_lang, $languages_array)) {
                    $filter_languages = false;
                }
            }

            $filter_groups = true;
            $filter_customers = true;
            if ($conf['groups'] !== '' && $conf['customers'] == '') {
                $groups_array = explode(';', $conf['groups']);
                foreach ($customer_groups as $group) {
                    if (!in_array($group, $groups_array)) {
                        $filter_groups = false;
                    } else {
                        $filter_groups = true;
                        break;
                    }
                }
                if (!$filter_groups) {
                    $filter_customers = false;
                }
            } else if ($conf['groups'] == '' && $conf['customers'] !== '') {
                $customers_array = explode(';', $conf['customers']);
                if (!in_array($id_customer, $customers_array)) {
                    $filter_customers = false;
                }
            } else if ($conf['groups'] !== '' && $conf['customers'] !== '') {
                $groups_array = explode(';', $conf['groups']);
                foreach ($customer_groups as $group) {
                    if (!in_array($group, $groups_array)) {
                        $filter_groups = false;
                    } else {
                        $filter_groups = true;
                    }
                }
                if (!$filter_groups) {
                    $customers_array = explode(';', $conf['customers']);
                    if (!in_array($id_customer, $customers_array)) {
                        $filter_customers = false;
                    } else {
                        $filter_customers = true;
                    }
                }
            }

            $filter_countries = true;
            if ($conf['countries'] !== '') {
                $countries_array = explode(';', $conf['countries']);

                if (!in_array($country->id, $countries_array)) {
                    $filter_countries = false;
                }
            }

            $filter_zones = true;
            if ($conf['zones'] !== '') {
                $zones_array = explode(';', $conf['zones']);
                if (!in_array($zone, $zones_array)) {
                    $filter_zones = false;
                }
            }
            $filter_categories = true;
            $filter_products = true;

            if (@unserialize($conf['categories']) !== false) {
                $categories_array = unserialize($conf['categories']);
            } else {
                $categories_array = explode(';', $conf['categories']);
            }

            if ($conf['categories'] !== '' && $conf['products'] == '' && $product_count_flag) {
                foreach ($categories as $category) {
                    if (in_array($category, $categories_array)) {
                        $filter_categories = true;
                        $filter_products = true;
                        break;
                    } else {
                        $filter_categories = false;
                    }
                }
                if (!$filter_categories) {
                    $filter_products = false;
                }
            } else if ($conf['categories'] == '' && $conf['products'] !== '' && $product_count_flag) {
                $products_array = explode(';', $conf['products']);
                if (!in_array($id_product, $products_array)) {
                    $filter_products = false;
                    $filter_categories = true;
                }
            } else if ($conf['categories'] !== '' && $conf['products'] !== '' && $product_count_flag) {
                foreach ($categories as $category) {
                    if (!in_array($category, $categories_array)) {
                        $filter_categories = false;
                    } else {
                        $filter_categories = true;
                        break;
                    }
                }
                if (!$filter_categories) {
                    $products_array = explode(';', $conf['products']);
                    if (!in_array($id_product, $products_array)) {
                        $filter_products = false;
                    } else {
                        $filter_products = true;
                    }
                } else {
                    $products_array = explode(';', $conf['products']);
                    if (!in_array($id_product, $products_array)) {
                        $filter_products = false;
                    }
                }
            }

            $filter_manufacturers = true;
            if ($conf['manufacturers'] !== '') {
                $manufacturers_array = explode(';', $conf['manufacturers']);
                if (!in_array($id_manufacturer, $manufacturers_array)) {
                    $filter_manufacturers = false;
                }
            }
            $filter_suppliers = true;
            if ($conf['suppliers'] !== '') {
                $suppliers_array = explode(';', $conf['suppliers']);
                if (!in_array($id_supplier, $suppliers_array)) {
                    $filter_suppliers = false;
                }
            }

            if ($product_count_flag && $filter_currencies && $filter_languages && $filter_attributes && $filter_features && $filter_groups && $filter_customers && $filter_countries && $filter_zones && $filter_categories && $filter_products && $filter_manufacturers && $filter_suppliers) {
                $array_configurations_result[] = $conf;
                if ($conf['first_condition']) {
                    break;
                } else {
                    continue;
                }
            }
        /*var_dump($id_product_attribute);
        var_dump($filter_groups);
        var_dump($filter_customers);
        var_dump($filter_countries);
        var_dump($filter_zones);
        var_dump($filter_categories);
        var_dump($filter_products);
        var_dump($filter_manufacturers);
        var_dump($filter_suppliers);
        var_dump($filter_attributes);
        var_dump($filter_features);*/
        }


        if (!empty($array_configurations_result)) {
            return $array_configurations_result;
        } else {
            return false;
        }
    }


    public function getGroupincReduction($giconfigurations, $id_product, $specific_price = false, $retail_price_with_taxes, $retail_price_without_taxes, $wholesale_price_with_taxes, $wholesale_price_without_taxes)
    {
        $groupinc_specific_price = array();
        $groupinc_quantity_fix = 0;
        $groupinc_quantity_percent = 0;
        $price_to_compare = 0;

        foreach ($giconfigurations as $gi) {
            $giconfig = new GroupincConfiguration($gi['id_groupinc_configuration']);

            if ($giconfig->threshold_min_price > 0 || $giconfig->threshold_max_price > 0) {
                if ($giconfig->threshold_price == 0) {
                    $price_to_compare = $wholesale_price_without_taxes;
                } else if ($giconfig->threshold_price == 1) {
                    $price_to_compare = $retail_price_without_taxes;
                } else if ($giconfig->threshold_price == 2) {
                    $price_to_compare = $wholesale_price_with_taxes;
                } else if ($giconfig->threshold_price == 3) {
                    $price_to_compare = $retail_price_with_taxes;
                }
            }

            if ($giconfig->price_application == 0) {
                $price_modified_return = $wholesale_price_without_taxes;
            } else if ($giconfig->price_application == 1) {
                $price_modified_return = $retail_price_without_taxes;
            } else if ($giconfig->price_application == 2) {
                $price_modified_return = $wholesale_price_with_taxes;
            } else if ($giconfig->price_application == 3) {
                $price_modified_return = $retail_price_with_taxes;
            }

            if ($giconfig->threshold_min_price > 0 && $giconfig->threshold_min_price >= $price_to_compare) {
                continue;
            }

            if ($giconfig->threshold_max_price > 0 && $giconfig->threshold_max_price <= $price_to_compare) {
                continue;
            }

            if ($giconfig->mode == 1) {
                $groupinc_specific_price['id_product'] = $id_product;
                $groupinc_specific_price['from_quantity'] = 1;
                $groupinc_specific_price['reduction_tax'] = 1;
                $groupinc_specific_price['price'] = -1.000000;
                $groupinc_specific_price['id_currency'] = 0;
                $groupinc_specific_price['id_product_attribute'] = 0;
                $groupinc_specific_price['label'] = $giconfig->name;

                if ($giconfig->override_discounts) {
                    if ($giconfig->type == 0) {
                        $groupinc_specific_price['reduction_type'] = 'amount';
                        $groupinc_specific_price['reduction'] = $giconfig->fix;
                    } else {
                        $groupinc_specific_price['reduction_type'] = 'percentage';
                        $groupinc_specific_price['reduction'] = $giconfig->percentage / 100;
                    }
                } else if (!$giconfig->skip_discounts && $specific_price) {
                    if ($giconfig->type == 0) {
                        $groupinc_specific_price['reduction_type'] = 'amount';
                        if ($specific_price['reduction_type'] == 'amount') {
                            $groupinc_specific_price['reduction'] = $specific_price['reduction'] + $giconfig->fix;
                        } else {
                            $groupinc_specific_price['reduction'] = $giconfig->fix;
                        }
                    } else {
                        $groupinc_specific_price['reduction_type'] = 'percentage';
                        if ($specific_price['reduction_type'] == 'percentage') {
                            $groupinc_specific_price['reduction'] = $specific_price['reduction'] + $giconfig->percentage / 100;
                        } else {
                            $groupinc_specific_price['reduction'] = $giconfig->percentage / 100;
                        }
                    }
                } else if ($giconfig->skip_discounts && $specific_price) {
                    $groupinc_specific_price = $specific_price;
                } else {
                    if ($giconfig->type == 0) {
                        $groupinc_specific_price['reduction_type'] = 'amount';
                        $groupinc_specific_price['reduction'] = $giconfig->fix;

                    } else {
                        $groupinc_specific_price['reduction_type'] = 'percentage';
                        $groupinc_specific_price['reduction'] = $giconfig->percentage / 100;
                        $price_modif = $price_modified_return * $groupinc_specific_price['reduction'];
                    }
                }

                if ($giconfig->type == 0) {
                    $price_modif = $price_modified_return - $groupinc_specific_price['reduction'];
                } else {
                    $price_modif = $price_modified_return * (1 - $groupinc_specific_price['reduction']);
                }

                if ($giconfig->min_result_price > 0) {
                    if ($price_modif < $giconfig->min_result_price) {
                        $groupinc_specific_price['reduction'] = Tools::ps_round(1 - ($giconfig->min_result_price / $price_modified_return), 4);
                    }
                }

                if ($giconfig->max_result_price > 0) {
                    if ($price_modif > $giconfig->max_result_price) {

                        $groupinc_specific_price['reduction'] = Tools::ps_round(1 - ($giconfig->max_result_price / $price_modified_return), 4);
                    }
                }

                if ($giconfig->first_condition) {
                    break;
                }
            }
        }
        return $groupinc_specific_price;
    }

    public function getPriceModified($id_giconfig, $retail_price_with_taxes, $retail_price_without_taxes, $wholesale_price_with_taxes, $wholesale_price_without_taxes)
    {
        // calculamos importes fijos
        $price_modified_return = 0;
        $groupinc_quantity_fix = 0;
        $groupinc_quantity_percent = 0;
        $giconfig = new GroupincConfiguration($id_giconfig);
        $price_to_compare = 0;

        if ($giconfig->threshold_min_price > 0 || $giconfig->threshold_max_price > 0) {
            if ($giconfig->threshold_price == 0) {
                $price_to_compare = $wholesale_price_without_taxes;
            } else if ($giconfig->threshold_price == 1) {
                $price_to_compare = $retail_price_without_taxes;
            } else if ($giconfig->threshold_price == 2) {
                $price_to_compare = $wholesale_price_with_taxes;
            } else if ($giconfig->threshold_price == 3) {
                $price_to_compare = $retail_price_with_taxes;
            }
        }

        if ($giconfig->price_application == 0) {
            $price_modified_return = $wholesale_price_without_taxes;
        } else if ($giconfig->price_application == 1) {
            $price_modified_return = $retail_price_without_taxes;
        } else if ($giconfig->price_application == 2) {
            $price_modified_return = $wholesale_price_with_taxes;
        } else if ($giconfig->price_application == 3) {
            $price_modified_return = $retail_price_with_taxes;
        }

        if ($giconfig->threshold_min_price > 0 && $giconfig->threshold_min_price > $price_to_compare) {
            return $price_modified_return;
        }

        if ($giconfig->threshold_max_price > 0 && $giconfig->threshold_max_price < $price_to_compare) {
            return $price_modified_return;
        }

        if ($giconfig->mode == 2) { // fixed price import
        	$price_modified_return = $giconfig->fix;
        } else {
	        if ($giconfig->type == 0) { // fix mode
	            if ($giconfig->mode == 0) {
	                $groupinc_quantity_fix += $giconfig->fix;
	            } else {
	                $groupinc_quantity_fix -= $giconfig->fix;
	            }
	            if ($giconfig->price_application == 0) {
	                $price_modified_return = $wholesale_price_without_taxes + $groupinc_quantity_fix;
	            } else if ($giconfig->price_application == 1) {
	                $price_modified_return = $retail_price_without_taxes + $groupinc_quantity_fix;
	            } else if ($giconfig->price_application == 2) {
	                $price_modified_return = $wholesale_price_with_taxes + $groupinc_quantity_fix;
	            } else if ($giconfig->price_application == 3) {
	                $price_modified_return = $retail_price_with_taxes + $groupinc_quantity_fix;
	            }
	        } else if ($giconfig->type == 1) { // percentage mode
	            if ($giconfig->price_calculation == 0) {
	                $groupinc_quantity_percent = $wholesale_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 1) {
                    $groupinc_quantity_percent = $retail_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 2) {
	                $groupinc_quantity_percent = $wholesale_price_with_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 3) {
	                //d($retail_price_with_taxes);
	                $groupinc_quantity_percent = $retail_price_with_taxes * $giconfig->percentage / 100;
	            }
                else if ($giconfig->price_calculation == 4) {
                    $groupinc_quantity_percent = ($retail_price_without_taxes - $wholesale_price_without_taxes) * $giconfig->percentage / 100;
                }
                else if ($giconfig->price_calculation == 5) {
                    $groupinc_quantity_percent = ($retail_price_with_taxes - $wholesale_price_with_taxes) * $giconfig->percentage / 100;
                }

	            if ($giconfig->price_application == 0) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $wholesale_price_without_taxes + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $wholesale_price_without_taxes - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 1) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $retail_price_without_taxes + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $retail_price_without_taxes - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 2) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $wholesale_price_with_taxes + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $wholesale_price_with_taxes - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 3) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $retail_price_with_taxes + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $retail_price_with_taxes - $groupinc_quantity_percent;
	                }
	            }
	        } else if ($giconfig->type == 2) { // fix + percentage mode
	            if ($giconfig->price_calculation == 0) {
	                $groupinc_quantity_percent = $wholesale_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 1) {
	                $groupinc_quantity_percent = $retail_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 2) {
	                $groupinc_quantity_percent = $wholesale_price_with_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 3) {
	                $groupinc_quantity_percent = $retail_price_with_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 4) {
                    $groupinc_quantity_percent = ($retail_price_without_taxes - $wholesale_price_without_taxes) * $giconfig->percentage / 100;
                } else if ($giconfig->price_calculation == 5) {
                    $groupinc_quantity_percent = ($retail_price_with_taxes - $wholesale_price_with_taxes) * $giconfig->percentage / 100;
                }

	            if ($giconfig->price_application == 0) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $wholesale_price_without_taxes + $giconfig->fix + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $wholesale_price_without_taxes - $giconfig->fix - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 1) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $retail_price_without_taxes + $giconfig->fix + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $retail_price_without_taxes - $giconfig->fix - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 2) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $wholesale_price_with_taxes + $giconfig->fix + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $wholesale_price_with_taxes - $giconfig->fix - $groupinc_quantity_percent;
	                }
	            } else if ($giconfig->price_application == 3) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $retail_price_with_taxes + $giconfig->fix + $groupinc_quantity_percent;
	                } else {
	                    $price_modified_return = $retail_price_with_taxes - $giconfig->fix - $groupinc_quantity_percent;
	                }
	            }
	/*        } else if ($giconfig->type == 3) { // percentage + fix mode
	            if ($giconfig->price_calculation == 0) {
	                $groupinc_quantity_percent = $wholesale_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 1) {
	                $groupinc_quantity_percent = $retail_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 2) {
	                $groupinc_quantity_percent = $wholesale_price_without_taxes * $giconfig->percentage / 100;
	            } else if ($giconfig->price_calculation == 3) {
	                $groupinc_quantity_percent = $retail_price_with_taxes * $giconfig->percentage / 100;
	            }
	            if ($giconfig->price_application == 2 and $use_taxes == 0) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $price + $groupinc_quantity_percent + $giconfig->fix;
	                } else {
	                    $price_modified_return = $price - $groupinc_quantity_percent - $giconfig->fix;
	                }
	            } else if ($giconfig->price_application == 3 and $use_taxes == 1) {
	                if ($giconfig->mode == 0) {
	                    $price_modified_return = $price + $groupinc_quantity_percent + $giconfig->fix;
	                } else {
	                    $price_modified_return = $price - $groupinc_quantity_percent - $giconfig->fix;
	                }
	            }*/
	        }
	    }
        if ($giconfig->min_result_price > 0) {

            if ($price_modified_return < $giconfig->min_result_price) {
                $price_modified_return = $giconfig->min_result_price;
            }
        }

        if ($giconfig->max_result_price > 0) {

            if ($price_modified_return > $giconfig->max_result_price) {
                $price_modified_return = $giconfig->max_result_price;
            }
        }

        return $price_modified_return;
/*
        $group_reduction = (($reduc = Group::getReductionByIdGroup($id_group)) != 0) ? ($price * $reduc / 100) : 0;
            }
            $price -= $group_reduction;
        }*/
    }

    public static function getProductsFromIdCart($id_cart)
    {
        $sql =
            'SELECT * FROM `'._DB_PREFIX_.'cart_product` WHERE id_cart = '.$id_cart;
        return Db::getInstance()->executeS($sql);

    }
}