<?php
/**
* Price increment/Reduction by groups, categories and more
*
* NOTICE OF LICENSE
*
* This product is licensed for one customer to use on one installation (test stores and multishop included).
* Site developer has the right to modify this module to suit their needs, but can not redistribute the module in
* whole or in part. Any other use of this module constitues a violation of the user agreement.
*
* DISCLAIMER
*
* NO WARRANTIES OF DATA SAFETY OR MODULE SECURITY
* ARE EXPRESSED OR IMPLIED. USE THIS MODULE IN ACCORDANCE
* WITH YOUR MERCHANT AGREEMENT, KNOWING THAT VIOLATIONS OF
* PCI COMPLIANCY OR A DATA BREACH CAN COST THOUSANDS OF DOLLARS
* IN FINES AND DAMAGE A STORES REPUTATION. USE AT YOUR OWN RISK.
*
*  @author    idnovate.com <info@idnovate.com>
*  @copyright 2016 idnovate.com
*  @license   See above
*/

class AdminGroupincController extends ModuleAdminController
{
    protected $delete_mode;

    protected $_defaultOrderBy = 'date_add';
    protected $_defaultOrderWay = 'DESC';
    protected $can_add_giconf = true;
    protected $top_elements_in_list = 4;
    protected $orderBy = 'id_product';
    protected $orderWay = 'ASC';

    protected static $meaning_status = array();

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'groupinc_configuration';
        $this->className = 'GroupincConfiguration';
        $this->tabClassName = 'AdminGroupinc';
        $this->lang = false;
        $this->addRowAction('edit');
        $this->addRowAction('delete');
        $this->_orderWay = $this->_defaultOrderWay;
        $this->taxes_included = (Configuration::get('PS_TAX') == '0' ? false : true);

        parent::__construct();

        $this->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'confirm' => $this->l('Delete selected items?'),
                'icon' => 'icon-trash'
            )
        );

        $this->context = Context::getContext();

        $this->default_form_language = $this->context->language->id;

        $this->fields_list = array(
            'id_groupinc_configuration' => array(
                'title' => $this->l('ID'),
                'align' => 'text-center',
                'class' => 'fixed-width-xs'
            ),
            'name' => array(
                'title' => $this->l('Name')
            ),
            'active' => array(
                'title' => $this->l('Enabled'),
                'align' => 'text-center',
                'active' => 'status',
                'type' => 'bool',
                'callback' => 'printActiveIcon'
            ),
            'backoffice' => array(
                'title' => $this->l('Backoffice'),
                'align' => 'text-center',
                'type' => 'bool',
                'callback' => 'printBackofficeIcon',
                'orderby' => false,
                'filter_key' => 'a!backoffice'
            ),
            'type' => array(
                'title' => $this->l('Type'),
                'callback' => 'getGroupincType',
                'align' => 'text-center'
            ),
            'mode' => array(
                'title' => $this->l('Mode'),
                'callback' => 'getGroupincMode',
                'align' => 'text-center'
            ),
            'fix' => array(
                'title' => $this->l('Fix'),
                'align' => 'text-center'
            ),
            'percentage' => array(
                'title' => $this->l('Percentage'),
                'align' => 'text-center'
            ),
            'suppliers' => array(
                'title' => $this->l('Supplier(s)'),
                'callback' => 'getSuppliers',
                'align' => 'text-center'
            ),
            'groups' => array(
                'title' => $this->l('Group(s)'),
                'callback' => 'getCustomerGroups',
                'align' => 'text-center'
            ),
            'customers' => array(
                'title' => $this->l('Customers(s)'),
                'callback' => 'getCustomers',
                'align' => 'text-center'
            ),
            'categories' => array(
                'title' => $this->l('Category(s)'),
                'callback' => 'getCategories',
                'align' => 'text-center'
            ),
            'products' => array(
                'title' => $this->l('Products(s)'),
                'callback' => 'getProducts',
                'align' => 'text-center'
            ),
            'currencies' => array(
                'title' => $this->l('Currency(es)'),
                'callback' => 'getCurrencies',
                'align' => 'text-center'
            ),
            'countries' => array(
                'title' => $this->l('Country(s)'),
                'callback' => 'getCountries',
                'align' => 'text-center'
            ),
            'zones' => array(
                'title' => $this->l('Zone(s)'),
                'callback' => 'getZones',
                'align' => 'text-center'
            ),
            'priority' => array(
                'title' => $this->l('Priority'),
                'align' => 'text-center'
            ),
        );

        $this->shopLinkType = 'shop';

        if (Shop::isFeatureActive() && (Shop::getContext() == Shop::CONTEXT_ALL || Shop::getContext() == Shop::CONTEXT_GROUP)) {
            $this->can_add_giconf = false;
        }
    }

    public function setMedia()
    {
        parent::setMedia();
        $this->addJqueryPlugin(array('typewatch', 'fancybox', 'autocomplete'));

        $this->addJqueryUI('ui.button');
        $this->addJqueryUI('effects.slide');
        $this->addJqueryUI('ui.sortable');
        $this->addJqueryUI('ui.droppable');
        $_path = _MODULE_DIR_.$this->module->name;

        $this->context->controller->addJS($_path.'/views/js/plugin/plugins/localisation/jquery.localisation.js');

        $this->context->controller->addCSS($_path.'/views/css/groupinc.css');
        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $this->context->controller->addJS($_path.'/views/js/gi_functions_15.js');
        } else {
            $this->context->controller->addJS($_path.'/views/js/gi_functions.js');
        }

        //$this->context->controller->addJS($_path.'/views/js/plugin/ui.multiselect.js');
    }

    public function postProcess()
    {
        return parent::postProcess();
    }

    public function initContent()
    {
        if ($this->action == 'select_delete') {
            $this->context->smarty->assign(array(
                'delete_form' => true,
                'url_delete' => htmlentities($_SERVER['REQUEST_URI']),
                'boxes' => $this->boxes,
            ));
        }

        $this->content .= $this->_createTemplate('admin_translations.tpl')->fetch();

        if (!$this->can_add_giconf && !$this->display) {
            $this->informations[] = $this->l('You have to select a shop if you want to create a new configuration.');
        }

        parent::initContent();
    }

    public function initToolbar()
    {
        parent::initToolbar();

        if (!$this->can_add_giconf) {
            unset($this->toolbar_btn['new']);
        }
    }

    public function getList($id_lang, $orderBy = null, $orderWay = null, $start = 0, $limit = null, $id_lang_shop = null)
    {
        parent::getList($id_lang, $orderBy, $orderWay, $start, $limit, $id_lang_shop);
    }

    public function initModal()
    {
        parent::initModal();

        $this->context->smarty->assign(array(
            'trad_link' => 'index.php?tab=AdminTranslations&token='.Tools::getAdminTokenLite('AdminTranslations').'&type=modules&lang=',
            'module_languages' => Language::getLanguages(false),
            'module_name' => 'codfee',
        ));

        $modal_content = $this->context->smarty->fetch('controllers/modules/modal_translation.tpl');
        $this->modals[] = array(
            'modal_id' => 'moduleTradLangSelect',
            'modal_class' => 'modal-sm',
            'modal_title' => $this->l('Translate this module'),
            'modal_content' => $modal_content
        );
    }

    public function initToolbarTitle()
    {
        parent::initToolbarTitle();

        switch ($this->display) {
            case '':
            case 'list':
                array_pop($this->toolbar_title);
                $this->toolbar_title[] = $this->l('Manage Groupinc Configuration');
                break;
            case 'view':
                if (($giconf = $this->loadObject(true)) && Validate::isLoadedObject($giconf)) {
                    array_pop($this->toolbar_title);
                    $this->toolbar_title[] = sprintf($this->l('Configuration: %s'), $giconf->name);
                }
                break;
            case 'add':
            case 'edit':
                array_pop($this->toolbar_title);
                if (($giconf = $this->loadObject(true)) && Validate::isLoadedObject($giconf)) {
                    $this->toolbar_title[] = sprintf($this->l('Editing Configuration: %s'), $giconf->name);
                } else {
                    $this->toolbar_title[] = $this->l('Creating a new Increment/Reduction configuration:');
                }
                break;
        }
    }

    public function initPageHeaderToolbar()
    {
        parent::initPageHeaderToolbar();

        if (empty($this->display)) {
            $this->page_header_toolbar_btn['desc-module-back'] = array(
                'href' => 'index.php?controller=AdminModules&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back'),
                'icon' => 'process-icon-back'
            );
            $this->page_header_toolbar_btn['desc-module-new'] = array(
                'href' => 'index.php?controller='.$this->tabClassName.'&add'.$this->table.'&token='.Tools::getAdminTokenLite($this->tabClassName),
                'desc' => $this->l('New'),
                'icon' => 'process-icon-new'
            );
            $this->page_header_toolbar_btn['desc-module-reload'] = array(
                'href' => 'index.php?controller='.$this->tabClassName.'&token='.Tools::getAdminTokenLite($this->tabClassName).'&reload=1',
                'desc' => $this->l('Reload'),
                'icon' => 'process-icon-refresh'
            );
            $this->page_header_toolbar_btn['desc-module-translate'] = array(
                'href' => '#',
                'desc' => $this->l('Translate'),
                'modal_target' => '#moduleTradLangSelect',
                'icon' => 'process-icon-flag'
            );
            $this->page_header_toolbar_btn['desc-module-hook'] = array(
                'href' => 'index.php?tab=AdminModulesPositions&token='.Tools::getAdminTokenLite('AdminModulesPositions').'&show_modules='.Module::getModuleIdByName('groupinc'),
                'desc' => $this->l('Manage hooks'),
                'icon' => 'process-icon-anchor'
            );
        }

        if (!$this->can_add_giconf) {
            unset($this->page_header_toolbar_btn['desc-module-new']);
        }
    }

    public function initProcess()
    {
        parent::initProcess();

        if (Tools::isSubmit('changeBackofficeVal') && $this->id_object) {
            if ($this->tabAccess['edit'] === '1') {
                $this->action = 'change_backoffice_val';
            } else {
                $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            }
        }
    }

    public function renderList()
    {
        if ((Tools::isSubmit('submitBulkdelete'.$this->table) || Tools::isSubmit('delete'.$this->table)) && $this->tabAccess['delete'] === '1') {
            $this->tpl_list_vars = array(
                'delete_groupincconf' => true,
                'REQUEST_URI' => $_SERVER['REQUEST_URI'],
                'POST' => $_POST
            );
        }
        return parent::renderList();
    }

    public function renderForm()
    {

        if (!($giconf = $this->loadObject(true))) {
            return;
        }

        $types = $this->getGroupincTypes();
        $modes = $this->getGroupincModes();
        $price_calc_options = $this->getGroupincPriceOptions(true);
        $price_options = $this->getGroupincPriceOptions();
        $products = array();
        $id_lang = 0;
        $id_shop = 0;
        $categories = array();

        if (version_compare(_PS_VERSION_, '1.5', '<')) {
        	$id_lang = (int)$this->context->cookie->id_lang;
        	$id_shop = (int)$this->context->cookie->id_shop;
        	$currencies = Currency::getCurrencies(false, true);
        } else {
        	$id_lang = (int)$this->context->language->id;
        	$id_shop = (int)$this->context->shop->id;
			if (Shop::isFeatureActive()) {
                $currencies = Currency::getCurrenciesByIdShop($this->context->shop->id);
            } else {
                $currencies = Currency::getCurrencies(false, true);
            }
        }

        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $categories = array_merge($categories, Category::getCategories((int)($this->context->cookie->id_lang), false, false, '', 'ORDER BY cl.`name` ASC'));
        }

        $groups = Group::getGroups($id_lang, true);
		$customers = array(); //Customer::getCustomers(true);
        $products = array();
		$countries = Country::getCountries($id_lang);
		$zones = Zone::getZones();
		$manufacturers = Manufacturer::getManufacturers(false, $id_lang, false);
		$suppliers = Supplier::getSuppliers(false, $id_lang, false);
		$num_products = $this->getNumProducts($id_lang, true, true);
		$languages = Language::getLanguages(false, $id_shop);
        $features = Feature::getFeatures((int)$id_lang);
        $attributeGroups = AttributeGroup::getAttributesGroups((int)$id_lang);

        $array_features = array();

        foreach ($features as $key => $feature) {
            if ($feature['name']) {
                $feature_values = FeatureValue::getFeatureValuesWithLang((int)$id_lang, $feature['id_feature']);
                if (!empty($feature_values)) {
                    $array_features[] = array(
                        'type' => 'select',
                        'label' => $this->l('Select').' '.$feature['name'],
                        'name' => 'feature_'.$feature['id_feature'].'[]',
                        'multiple' => true,
                        'required' => false,
                        'class' => 'multiple_select',
                        'col' => '3',
                        'options' => array(
                            'query' => $feature_values,
                            'id' => 'id_feature_value',
                            'name' => 'value'
                        ),
                        'desc' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? $this->l('Select the ').' '.$feature['name'].' '.$this->l('to apply this configuration') : '',
                    );
                }
            }
        }

        $array_attributes = array();
        foreach ($attributeGroups as $key => $attributeGroup) {
            if ($attributeGroup['name']) {
                if (!empty($attributeGroups)) {
                    $array_attributes[] = array(
                        'type' => 'select',
                        'label' => $this->l('Select').' '.$attributeGroup['name'],
                        'name' => 'attribute_'.$attributeGroup['id_attribute_group'].'[]',
                        'multiple' => true,
                        'required' => false,
                        'class' => 'multiple_select',
                        'col' => '3',
                        'options' => array(
                            'query' => AttributeGroup::getAttributes((int)$id_lang, $attributeGroup['id_attribute_group']),
                            'id' => 'id_attribute',
                            'name' => 'name'
                        ),
                        'desc' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? $this->l('Select the ').' '.$attributeGroup['name'].' '.$this->l('to apply this configuration') : '',
                    );
                }
            }
        }

        $products = array();

        $this->multiple_fieldsets = true;
        $this->default_form_language = $this->context->language->id;

        $this->fields_form[]['form'] = array(
            'legend' => array(
                'title' => $this->l('Increment/Reduction Configuration'),
                'icon' => 'icon-key'
            ),
            'input' => array(
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Active'),
                    'name' => 'active',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Enable or Disable this configuration'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Name'),
                    'name' => 'name',
                    'required' => true,
                    'col' => '5',
                    'desc' => $this->l('Invalid characters:').' !&lt;&gt;,;?=+()@#"°{}_$%:',
                ),
                array(
                    'type' => 'date',
                    'label' => $this->l('Date From'),
                    'name' => 'date_from',
                    'col' => '4',
                    'desc' => $this->l('Date from which the rule is valid. You can use hours, minutes and secons. Example: 2016-10-27 is considered 2016-10-27 00:00:00 and it means that the rule is valid from 2016-10-27 00:00:00'),
                ),
                array(
                    'type' => 'date',
                    'label' => $this->l('Date To'),
                    'name' => 'date_to',
                    'col' => '4',
                    'desc' => $this->l('Date to which the rule is valid. You can use hours, minutes and secons. Example: 2016-10-27 is considered 2016-10-27 00:00:00 and it means that the rule is valid until 2016-10-26 23:59:59'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Mode'),
                    'name' => 'mode',
                    'required' => true,
                    'col' => '5',
                    'options' => array(
                        'query' => $modes,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => array(
                            'value' => '',
                            'label' => $this->l('-- Choose --')
                        )
                    ),
                    'desc' => $this->l('Select if you want to configure an Increment, a Reduction or a Fixed Price'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Type'),
                    'name' => 'type',
                    'required' => true,
                    'col' => '5',
                    'options' => array(
                        'query' => $types,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => array(
                            'value' => '',
                            'label' => $this->l('-- Choose --')
                        )
                    ),
                    'desc' => $this->l('Select the type that you want to apply, percentage, fix import or a combination of fix + percentage'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Price percentage calculation'),
                    'name' => 'price_calculation',
                    'required' => true,
                    'col' => '5',
                    'class' => 'price_calculation',
                    'options' => array(
                        'query' => $price_calc_options,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => array(
                            'value' => '',
                            'label' => $this->l('-- Choose --')
                        )
                    ),
                    'desc' => $this->l('Select the price from where you want to calculate the percentage of the Increment/Reduction'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Price to apply the Increment/Reduction'),
                    'name' => 'price_application',
                    'required' => true,
                    'col' => '5',
                    'options' => array(
                        'query' => $price_options,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => array(
                            'value' => '',
                            'label' => $this->l('-- Choose --')
                        )
                    ),
                    'desc' => $this->l('Select the price where you want to apply the Increment/Reduction. The amount (percentage or fix) will be added, subtract or established to this price'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Fix'),
                    'name' => 'fix',
                    'col' => '5',
                    'desc' => sprintf($this->l('Fix Increment/Reduction (by default currency) %s'), ($this->taxes_included) ? $this->l('taxes included') : $this->l('without taxes')),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Percentage'),
                    'name' => 'percentage',
                    'col' => '1',
                    'suffix' => '%',
                    'desc' => $this->l('Percentage amount'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Minimum result price'),
                    'name' => 'min_result_price',
                    'col' => '5',
                    'default' => '0',
                    'desc' => $this->l('Set a minimum result price if the calculated price of product is lower than this. If this value is 0 there\'s no minimum result price'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Maximum result price'),
                    'name' => 'max_result_price',
                    'col' => '5',
                    'default' => '0',
                    'desc' => $this->l('Set a maximum result price if the calculated price of product is greater than this. If this value is 0 there\'s no maximum result price'),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Price to calculate the threshold'),
                    'name' => 'threshold_price',
                    'col' => '5',
                    'options' => array(
                        'query' => $price_options,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => array(
                            'value' => '',
                            'label' => $this->l('-- Choose --')
                        )
                    ),
                    'desc' => $this->l('Select the price which the threshold will be calculated. For example: Appy the rule to the products that have a Retail price with taxes prices between 10€ (minimum) and 50€ (maximum)'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Threshold minimum price'),
                    'name' => 'threshold_min_price',
                    'col' => '5',
                    'default' => '0',
                    'desc' => sprintf($this->l('Minimum price to set a less Threshold price (by default currency with %s) to apply the rule. If this value is 0 there\'s no minimum limit'), ($this->taxes_included) ? $this->l('taxes included') : $this->l('without taxes')),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Threshold maximum price'),
                    'name' => 'threshold_max_price',
                    'col' => '5',
                    'default' => '0',
                    'desc' => sprintf($this->l('Maximum price to set a top Threshold price (by default currency with %s) to apply the rule. If this value is 0 there\'s no maximum limit'), ($this->taxes_included) ? $this->l('taxes included') : $this->l('without taxes')),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Product quantity units'),
                    'name' => 'product_qty',
                    'col' => '4',
                    'default' => '0',
                    'desc' => sprintf($this->l('Minimum quantity of units of products in cart')),
                ),
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Show Reduction as discount'),
                    'name' => 'show_as_discount',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'show_as_discount_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'show_as_discount_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Show the reduction in the price like a discount (specific price). Only enable it if you have configured a Reduction'),
                ),
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Skip other discounts'),
                    'name' => 'skip_discounts',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'skip_discounts_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'skip_discounts_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('If the product has a previous discount, the rule will not be applied, will be skipped'),
                ),
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Override discounts'),
                    'name' => 'override_discounts',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'override_discounts_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'override_discounts_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('If the product has a previous discount, the rule will be applied and the previous discount will be ommited (not deleted)'),
                ),
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Apply only one condition?'),
                    'name' => 'first_condition',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'first_condition_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'first_condition_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('If you enable this option, when this rule is applied, the other rules with low priority will be ommited'),
                ),
                array(
                    'type' => (version_compare(_PS_VERSION_, '1.6', '>=')) ? 'switch' : 'radio',
                    'label' => $this->l('Use this configuration in Admin (Backoffice)'),
                    'name' => 'backoffice',
                    'class' => 't',
                    'col' => '5',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'backoffice_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'backoffice_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Use this configuration in Prestashop Backoffice to show the prices of products modified with this conditions'),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Priority'),
                    'name' => 'priority',
                    'default' => '1',
                    'col' => '5',
                    'desc' => $this->l('Set priority when 2 or more configurations overlaps. Configuration with less number here will have more priority'),
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'type' => 'submit',
            ),
        );

        $this->fields_form[]['form'] = array(
            'legend' => array(
                'title' => $this->l('Conditions and filters'),
                'icon' => 'icon-key'
            ),
            'input' => array(
                ),
            );

        if (version_compare(_PS_VERSION_, '1.6', '>=')) {
            $selected_categories = array();
            if ($giconf->categories != '') {
                if (@unserialize($giconf->categories) !== false) {
                    $selected_categories = unserialize($giconf->categories);
                } else {
                    $selected_categories = explode(';', $giconf->categories);
                }
            }

            $categories_form_array = array(
               'type'  => 'categories',
               'label' => $this->l('Select Category(s)'),
               'multiple' => true,
               'name'  => 'categories',
               'col' => '5',
               'tree'  => array(
                    'id' => 'id_category',
                    'use_checkbox' => true,
                    'selected_categories' => $selected_categories,
                    ),
                'desc' => $this->l('Select the Category(es) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Categories'),
            );
        } else {
            $categories_form_array = array(
                'type' => 'select',
                'label' => $this->l('Select Category(s)'),
                'class' => 'multiple_select',
                'name'  => 'categories[]',
                'multiple' => true,
                'required' => false,
                'col' => '2',
                'options' => array(
                    'query' => $categories,
                    'id' => 'id_category',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Category(es) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Categories'),
            );
        }

        array_push($this->fields_form[1]['form']['input'], $categories_form_array);

        $render_form_end = array();

        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Product(s)'),
                'name' => 'products[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $products,
                    'id' => 'id_product',
                    'name' => 'name'
                ),
                //'desc' => $disable_products ? $this->l('Warning: Products filter disabled due to the number of products') : $this->l('Select the Product(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Products'),
                'desc' => $this->l('Select the Product(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Products'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Currency(es)'),
                'name' => 'currencies[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $currencies,
                    'id' => 'id_currency',
                    'name' => 'name'
                ),
                'desc' => $this->l('Currency(es) to apply this configuration'),
            );

        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Language(s)'),
                'name' => 'languages[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $languages,
                    'id' => 'id_lang',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Language(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Languages'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Customer group(s)'),
                'name' => 'groups[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $groups,
                    'id' => 'id_group',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Customer Group(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Groups'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Customer(s)'),
                'name' => 'customers[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $customers,
                    'id' => 'id_customer',
                    'name' => 'email'
                ),
                'desc' => $this->l('Select the Customer(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Customers'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Country(s)'),
                'name' => 'countries[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $countries,
                    'id' => 'id_country',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Country(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Countries'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Zone(s)'),
                'name' => 'zones[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $zones,
                    'id' => 'id_zone',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Zone(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Zones'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Manufacturer(s)'),
                'name' => 'manufacturers[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $manufacturers,
                    'id' => 'id_manufacturer',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Manufacturer(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Manufacturers'),
            );
        $render_form_end[] =
            array(
                'type' => 'select',
                'label' => $this->l('Select Supplier(s)'),
                'name' => 'suppliers[]',
                'class' => 'multiple_select',
                'multiple' => true,
                'required' => false,
                'col' => '5',
                'options' => array(
                    'query' => $suppliers,
                    'id' => 'id_supplier',
                    'name' => 'name'
                ),
                'desc' => $this->l('Select the Supplier(s) where the rule will be applied. If you don\'t select any value, the rule will be applied to all Suppliers'),
            );


        foreach ($array_features as $f) {
            array_push($render_form_end, $f);
        }
        foreach ($array_attributes as $a) {
            array_push($render_form_end, $a);
        }

        // add the final part of the form
        foreach ($render_form_end as $f) {
            array_push($this->fields_form[1]['form']['input'], $f);
        }

        $this->fields_form[]['form']['submit'] = array(
                'title' => $this->l('Save'),
                'type' => 'submit',
        );

        //Load db values for select inputs
        if ($giconf->id) {
            $groups_db = explode(';', $giconf->groups);
            $zones_db = explode(';', $giconf->zones);
            $countries_db = explode(';', $giconf->countries);
            $manufacturers_db = explode(';', $giconf->manufacturers);
            $products_db = explode(';', $giconf->products);
            $customers_db = explode(';', $giconf->customers);
            $suppliers_db = explode(';', $giconf->suppliers);
            $currencies_db = explode(';', $giconf->currencies);
            $languages_db = explode(';', $giconf->languages);
            $attributes_db = Tools::jsonDecode($giconf->attributes);

            $features_decoded_array = Tools::jsonDecode($giconf->features);
            $attributes_decoded_array = Tools::jsonDecode($giconf->attributes);

            $this->fields_value = array(
                'groups[]' => $groups_db,
                'zones[]' => $zones_db,
                'countries[]' => $countries_db,
                'manufacturers[]' => $manufacturers_db,
                /*'products[]' => $products_db,
                'customers[]' => $customers_db, */
                'suppliers[]' => $suppliers_db,
                'currencies[]' => $currencies_db,
                'languages[]' => $languages_db,
            );

            $i = 0;
            if (!empty($features_decoded_array)) {
                foreach($features_decoded_array as $key => $feature) {
                    $feature_values['feature_'.$key.'[]'] = explode(';',$feature);
                    $i++;
                }

                foreach ($feature_values as $key => $f) {
                    $this->fields_value[$key] = $f;
                }
            }

            $i = 0;
            if (!empty($attributes_decoded_array)) {
                foreach($attributes_decoded_array as $key => $attribute) {
                    $attribute_values['attribute_'.$key.'[]'] = explode(';',$attribute);
                    $i++;
                }

                foreach ($attribute_values as $key => $f) {
                    $this->fields_value[$key] = $f;
                }
            }

            if (version_compare(_PS_VERSION_, '1.6', '<')) {
                $this->fields_value['categories[]'] = explode(';', $giconf->categories);
            }
        }

        $products = $this->getProductsLite($id_lang, true, true);
        $customers = Customer::getCustomers(true);

        $this->context->smarty->assign(array(
            'products_selected' => $giconf->products,
            'customers_selected' => $giconf->customers,
            'customers_available' => $customers,
            'products_available' => $products,
        ));

        $this->content .= parent::renderForm();
        $this->content .= $this->_createTemplate('admin_content.tpl')->fetch();
        return;
    }

    public function renderView()
    {
        return parent::renderView();
    }

    public function processDelete()
    {
        parent::processDelete();
    }

    public function processSave()
    {
        if (Tools::getValue('submitFormAjax')) {
            $this->redirect_after = false;
        }

        if ($this->_formValidations()) {
            $_POST['groups'] = (!Tools::getValue('groups')) ? '' : implode(';', Tools::getValue('groups'));
            $_POST['countries'] = (!Tools::getValue('countries')) ? '' : implode(';', Tools::getValue('countries'));
            $_POST['zones'] = (!Tools::getValue('zones')) ? '' : implode(';', Tools::getValue('zones'));
            $_POST['manufacturers'] = (!Tools::getValue('manufacturers')) ? '' : implode(';', Tools::getValue('manufacturers'));
            $_POST['suppliers'] = (!Tools::getValue('suppliers')) ? '' : implode(';', Tools::getValue('suppliers'));
            $_POST['customers'] = (!Tools::getValue('customers')) ? '' : implode(';', Tools::getValue('customers'));
            $_POST['products'] = (!Tools::getValue('products')) ? '' : implode(';', Tools::getValue('products'));
            $_POST['currencies'] = (!Tools::getValue('currencies')) ? '' : implode(';', Tools::getValue('currencies'));
            $_POST['languages'] = (!Tools::getValue('languages')) ? '' : implode(';', Tools::getValue('languages'));
            $_POST['attributes'] = (!Tools::getValue('attributes')) ? '' : implode(';', Tools::getValue('attributes'));
            $_POST['features'] = (!Tools::getValue('features')) ? '' : implode(';', Tools::getValue('features'));

            if (version_compare(_PS_VERSION_, '1.6', '>=')) {
                if (Tools::isSubmit('categories')) {
                    $cats = Tools::getValue('categories');
                    $_POST['categories'] = serialize($cats);
                } else {
                    $_POST['categories'] = '';
                }
            } else {
                $_POST['categories'] = (!Tools::isSubmit('categories')) ? '' : implode(';', Tools::getValue('categories'));
            }

            if (Tools::getValue('type') == 1) {
                $_POST['fix'] = 0;
            } else if (Tools::getValue('type') == 0) {
                $_POST['percentage'] = 0;
            }

            if (Tools::getValue('mode') == 0) {
                $_POST['show_as_discount'] = 0;
            }

            if (version_compare(_PS_VERSION_, '1.6', '>=')) {
                $this->cleanCache();
            }
            return parent::processSave();
        }
    }

    protected function afterAdd($object)
    {
        $id = Tools::getValue('id_groupinc_configuration');
        $this->afterUpdate($object, $id);
        return true;
    }

    protected function afterUpdate($object, $id = false)
    {
        if ($id) {
            $giconf = new GroupincConfiguration((int)$id);
        } else {
            $giconf = new GroupincConfiguration((int)$object->id);
        }
        if (Validate::isLoadedObject($giconf)) {
            $features = Feature::getFeatures((int)$this->context->cookie->id_lang);
            $attributeGroups = AttributeGroup::getAttributesGroups((int)$this->context->cookie->id_lang);

            $array_features_result = array();
            $array_attributes_result = array();

            foreach ($features as $f) {
                if (Tools::getValue('feature_'.$f['id_feature'])) {
                    $array_features_result[$f['id_feature']] = implode(';', Tools::getValue('feature_'.$f['id_feature']));
                }
            }

            foreach ($attributeGroups as $a) {
                if (Tools::getValue('attribute_'.$a['id_attribute_group'])) {
                    $array_attributes_result[$a['id_attribute_group']] = implode(';', Tools::getValue('attribute_'.$a['id_attribute_group']));
                }
            }

            $giconf->features = Tools::jsonEncode($array_features_result);
            $giconf->attributes = Tools::jsonEncode($array_attributes_result);
            $giconf->save();
        }
        return true;
    }

    /**
     * @param string $token
     * @param int $id
     * @param string $name
     * @return mixed
     */
    public function displayDeleteLink($token = null, $id, $name = null)
    {
        $tpl = $this->createTemplate('helpers/list/list_action_delete.tpl');

        $tpl->assign(array(
            'href' => self::$currentIndex.'&'.$this->identifier.'='.$id.'&delete'.$this->table.'&token='.($token != null ? $token : $this->token),
            'confirm' => $this->l('Delete the selected item? ').$name,
            'action' => $this->l('Delete'),
            'id' => $id,
        ));

        return $tpl->fetch();
    }

    protected function getGroupincTypes()
    {
        //$types = array($this->l('Fix'), $this->l('Percentage'), $this->l('Fix + Percentage'), $this->l('Percentage + Fix'));
        $types = array($this->l('Fix'), $this->l('Percentage'), $this->l('Fix + Percentage'));

        $list_types = array();
        foreach ($types as $key => $type) {
            $list_types[$key]['id'] = $key;
            $list_types[$key]['value'] = $key;
            $list_types[$key]['name'] = $type;
        }
        return $list_types;
    }

    protected function getGroupincModes()
    {
        $modes = array($this->l('Increment'), $this->l('Reduction'), $this->l('Set Fixed price'));

        $list_modes = array();
        foreach ($modes as $key => $mode) {
            $list_modes[$key]['id'] = $key;
            $list_modes[$key]['value'] = $key;
            $list_modes[$key]['name'] = $mode;
        }
        return $list_modes;
    }

    public function getGroupincType($type)
    {
        if ($type == '0') {
            return $this->l('Fix');
        } elseif ($type == '1') {
            return $this->l('Percentage');
        } else if ($type == '2')
            return $this->l('Fix + Percentage');
    }

    public function getGroupincMode($mode)
    {
        if ($mode == '0') {
            return $this->l('Increment');
        } elseif ($mode == '1') {
            return $this->l('Reduction');
        }  elseif ($mode == '2') {
			 return $this->l('Fixed Price');
		}
    }

    public function getGroupincPriceOptions($calculation = false)
    {
        $price_options = array($this->l('Wholesale Price without Taxes'), $this->l('Retail Price Without Taxes'), $this->l('Wholesale Price with Taxes'), $this->l('Retail Price with Taxes'));

        if ($calculation) {
            array_push($price_options, $this->l('Benefits margin without taxes (Retail - Wholesale)'));
            array_push($price_options, $this->l('Benefits margin with taxes (Retail - Wholesale)'));
        }

        $list_price_options = array();
        foreach ($price_options as $key => $mode) {
            $list_price_options[$key]['id'] = $key;
            $list_price_options[$key]['value'] = $key;
            $list_price_options[$key]['name'] = $mode;
        }
        return $list_price_options;
    }

    public function getCustomerGroups($ids_customer_groups)
    {
        if ($ids_customer_groups === '' || $ids_customer_groups === 'all') {
            return $this->l('All');
        }
        $groups = array();
        $groups_array = explode(';', $ids_customer_groups);
        foreach ($groups_array as $key => $group) {
            if ($key == $this->top_elements_in_list) {
                $groups[] = $this->l('...and more');
                break;
            }
            $group = new Group($group, $this->context->language->id);
            $groups[] = $group->name;
        }
        return implode('<br />', $groups);
    }

    public function getCountries($ids_countries)
    {
        if ($ids_countries === '' || $ids_countries === 'all') {
            return $this->l('All');
        }
        $countries = array();
        $countries_array = explode(';', $ids_countries);
        foreach ($countries_array as $key => $country) {
            if ($key == $this->top_elements_in_list) {
                $countries[] = $this->l('...and more');
                break;
            }
            $country = new Country($country, $this->context->language->id);
            $countries[] = $country->name;
        }
        return implode('<br />', $countries);
    }

    public function getZones($ids_zones)
    {
        if ($ids_zones === '' || $ids_zones === 'all') {
            return $this->l('All');
        }
        $zones = array();
        $zones_array = explode(';', $ids_zones);
        foreach ($zones_array as $key => $zone) {
            if ($key == $this->top_elements_in_list) {
                $zones[] = $this->l('...and more');
                break;
            }
            $zone = new Zone($zone, $this->context->language->id);
            $zones[] = $zone->name;
        }
        return implode('<br />', $zones);
    }

    public function getSuppliers($ids_suppliers)
    {
        if ($ids_suppliers === '' || $ids_suppliers === 'all') {
            return $this->l('All');
        }

        $suppliers = array();
        $suppliers_array = explode(';', $ids_suppliers);
        foreach ($suppliers_array as $key => $supplier) {
            if ($key == $this->top_elements_in_list) {
                $suppliers[] = $this->l('...and more');
                break;
            }
            $supplier = new Supplier($supplier);
            $suppliers[] = $supplier->name;
        }
        return implode('<br />', $suppliers);
    }


    public function getCategories($ids_categories)
    {
        if ($ids_categories === '' || $ids_categories === 'all') {
            return $this->l('All');
        }

        $categories = array();

		if (@unserialize($ids_categories) !== false) {
			$categories_array = unserialize($ids_categories);
        } else {
			$categories_array = explode(';', $ids_categories);
        }

        foreach ($categories_array as $key => $category) {
            if ($key == $this->top_elements_in_list) {
                $categories[] = $this->l('...and more');
                break;
            }
            $category = new Category($category, $this->context->language->id);
            $categories[] = $category->name;
        }
        return implode('<br />', $categories);
    }

    public function getCurrencies($ids_currencies)
    {
        if ($ids_currencies === '' || $ids_currencies === 'all') {
            return $this->l('All');
        }
        $currencies = array();
        $currencies_array = explode(';', $ids_currencies);
        foreach ($currencies_array as $key => $currency) {
            if ($key == $this->top_elements_in_list) {
                $currencies[] = $this->l('...and more');
                break;
            }
            $currency = new Currency($currency);
            $currencies[] = $currency->name;
        }
        return implode('<br />', $currencies);
    }

    public function getProducts($ids_products)
    {
        if ($ids_products === '' || $ids_products === 'all') {
            return $this->l('All');
        }
        $products = array();
        $products_array = explode(';', $ids_products);
        foreach ($products_array as $key => $product) {
            if ($key == $this->top_elements_in_list) {
                $products[] = $this->l('...and more');
                break;
            }
            $product = new Product($product, $this->context->language->id);
            $products[] = '['.$product->id.'] - '.$product->name[$this->context->language->id];
        }
        return implode('<br />', $products);
    }

    public function getCustomers($ids_customers)
    {
        if ($ids_customers === '' || $ids_customers === 'all') {
            return $this->l('All');
        }
        $customers = array();
        $customers_array = explode(';', $ids_customers);
        foreach ($customers_array as $key => $customer) {
            if ($key == $this->top_elements_in_list) {
                $customers[] = $this->l('...and more');
                break;
            }
            $customer = new Customer($customer, $this->context->language->id);
            $customers[] = $customer->firstname.' '.$customer->lastname;
        }
        return implode('<br />', $customers);
    }

    private function _createTemplate($tpl_name)
    {
        if ($this->override_folder) {
            if ($this->context->controller instanceof ModuleAdminController) {
                $override_tpl_path = $this->context->controller->getTemplatePath().$tpl_name;
            } elseif ($this->module) {
                $override_tpl_path = _PS_MODULE_DIR_.$this->module_name.'/views/templates/admin/'.$tpl_name;
            } else {
                if (file_exists($this->context->smarty->getTemplateDir(1).DIRECTORY_SEPARATOR.$this->override_folder.$this->base_folder.$tpl_name)) {
                    $override_tpl_path = $this->context->smarty->getTemplateDir(1).DIRECTORY_SEPARATOR.$this->override_folder.$this->base_folder.$tpl_name;
                } elseif (file_exists($this->context->smarty->getTemplateDir(0).DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.$this->override_folder.$this->base_folder.$tpl_name)) {
                    $override_tpl_path = $this->context->smarty->getTemplateDir(0).'controllers'.DIRECTORY_SEPARATOR.$this->override_folder.$this->base_folder.$tpl_name;
                }
            }
        } else if ($this->module) {
            $override_tpl_path = _PS_MODULE_DIR_.$this->module_name.'/views/templates/admin/'.$tpl_name;
        }
        if (isset($override_tpl_path) && file_exists($override_tpl_path)) {
            return $this->context->smarty->createTemplate($override_tpl_path, $this->context->smarty);
        } else {
            return $this->context->smarty->createTemplate($tpl_name, $this->context->smarty);
        }
    }

    private function _formValidations()
    {
        if (trim(Tools::getValue('name')) == '') {
            $this->validateRules();
            $this->errors[] = Tools::displayError($this->l('Field Name can not be empty.'));
            $this->display = 'edit';
        }

        if (Tools::getValue('date_from') > 0) {
	        if (!Validate::isDate(Tools::getValue('date_from'))) {
	            $this->errors[] = $this->l('Invalid "Date From" format');
	            $this->display = 'edit';
	        }
	    }

	    if (Tools::getValue('date_to') > 0) {
	        if (!Validate::isDate(Tools::getValue('date_to'))) {
	            $this->errors[] = $this->l('Invalid "Date To" format');
	            $this->display = 'edit';
	        }
	    }

        if (count($this->errors)) {
            return false;
        }

        return true;
    }

    public function processChangeBackofficeVal()
    {
        $gi = new GroupincConfiguration($this->id_object);
        if (!Validate::isLoadedObject($gi)) {
            $this->errors[] = Tools::displayError('An error occurred while updating Virtual POS information.');
        }
        $gi->backoffice = $gi->backoffice ? 0 : 1;
        if (!$gi->update()) {
            $this->errors[] = Tools::displayError('An error occurred while updating customer information.');
        }
        Tools::redirectAdmin(self::$currentIndex.'&token='.$this->token);
    }

    public function printBackofficeIcon($value, $gi)
    {
        return '<a class="list-action-enable '.($value ? 'action-enabled' : 'action-disabled').'" href="index.php?'.htmlspecialchars('tab=AdminGroupinc&id_groupinc_configuration='.(int)$gi['id_groupinc_configuration'].'&changeBackofficeVal&token='.Tools::getAdminTokenLite('AdminGroupinc')).'">
                '.($value ? '<i class="icon-check"></i>' : '<i class="icon-remove"></i>').
            '</a>';
    }

    protected function getProductsLite($id_lang, $only_active = false, $front = false, Context $context = null)
    {
        if (!$context)
            $context = Context::getContext();

        $sql = 'SELECT p.`id_product`, CONCAT(p.`reference`, " - ", pl.`name`) as name FROM `'._DB_PREFIX_.'product` p
                '.Shop::addSqlAssociation('product', 'p').'
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (p.`id_product` = pl.`id_product` '.Shop::addSqlRestrictionOnLang('pl').')
                WHERE pl.`id_lang` = '.(int)$id_lang.
                    ($front ? ' AND product_shop.`visibility` IN ("both", "catalog")' : '').
                    ($only_active ? ' AND product_shop.`active` = 1' : '');

        $rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return ($rq);
    }

    protected function getNumProducts($id_lang, $only_active = false, $front = false, Context $context = null)
    {
        if (!$context)
            $context = Context::getContext();

        $sql = 'SELECT count(p.`id_product`) as num_products FROM `'._DB_PREFIX_.'product` p
                '.Shop::addSqlAssociation('product', 'p').'
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (p.`id_product` = pl.`id_product` '.Shop::addSqlRestrictionOnLang('pl').')
                WHERE pl.`id_lang` = '.(int)$id_lang.
                    ($front ? ' AND product_shop.`visibility` IN ("both", "catalog")' : '').
                    ($only_active ? ' AND product_shop.`active` = 1' : '');

        $rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);
        return ($rq);
    }

    protected function cleanCache() {
        /* delete smarty cache to refresh the static blocks */
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        if (version_compare(_PS_VERSION_, '1.7', '<') && version_compare(_PS_VERSION_, '1.4', '>')) {
            PrestaShopAutoload::getInstance()->generateIndex();
        } else if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            Tools::generateIndex();
        }
    }
}
